@@ -1,6 +1,5 @@
-/* -*- c++ -*- */
 /*
- * Copyright 2010 Ettus Research LLC
+ * Copyright 2010-2012 Ettus Research LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +18,7 @@
 //Routines to handle updating the SPI Flash firmware via UDP
 
 #include <net_common.h>
+#include "memory_map.h"
 #include "usrp2/fw_common.h"
 #include "spi.h"
 #include "spi_flash.h"
