@@ -1,5 +1,5 @@
 //
-// Copyright 2011 Ettus Research LLC
+// Copyright 2011-2012 Ettus Research LLC
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -29,6 +29,9 @@ namespace uhd{
     //! Get a string representing the system's appdata directory
     UHD_API std::string get_app_path(void);
 
+    //! Get a string representing the system's pkg data directory
+    UHD_API std::string get_pkg_data_path(void);
+
 } //namespace uhd
 
 #endif /* INCLUDED_UHD_UTILS_PATHS_HPP */
