@@ -16,45 +16,64 @@
 //
 
 #include <uhd/transport/usb_zero_copy.hpp>
-#include <uhd/transport/bounded_buffer.hpp>
 #include <uhd/transport/buffer_pool.hpp>
 #include <uhd/utils/byteswap.hpp>
 #include <uhd/utils/msg.hpp>
+#include <uhd/utils/tasks.hpp>
+#include <uhd/utils/atomic.hpp>
 #include <boost/foreach.hpp>
+#include <boost/make_shared.hpp>
+#include <boost/thread/mutex.hpp>
+#include <boost/thread/condition_variable.hpp>
+#include <boost/bind.hpp>
 #include <vector>
 #include <iostream>
 
+using namespace uhd;
 using namespace uhd::transport;
 
+static const boost::posix_time::time_duration AUTOFLUSH_TIMEOUT(boost::posix_time::milliseconds(1));
+
 /***********************************************************************
  * USB zero copy wrapper - managed receive buffer
  **********************************************************************/
 class usb_zero_copy_wrapper_mrb : public managed_recv_buffer{
 public:
-    usb_zero_copy_wrapper_mrb(bounded_buffer<usb_zero_copy_wrapper_mrb *> &queue):
-        _queue(queue){/*NOP*/}
+    usb_zero_copy_wrapper_mrb(void){/*NOP*/}
 
     void release(void){
-        if (not _mrb) return;
         _mrb.reset(); //decrement ref count, other MRB's may hold a ref
-        _queue.push_with_haste(this);
+        _claimer.release();
     }
 
-    UHD_INLINE sptr get_new(managed_recv_buffer::sptr mrb, const void *mem, size_t len){
+    UHD_INLINE sptr get_new(
+        managed_recv_buffer::sptr &mrb, size_t &offset_bytes,
+        const double timeout, size_t &index
+    ){
+        if (not mrb or not _claimer.claim_with_wait(timeout)) return sptr();
+
+        index++; //advances the caller's buffer
+
+        //hold a copy of the buffer shared pointer
         _mrb = mrb;
-        _mem = mem;
-        _len = len;
-        return make_managed_buffer(this);
+
+        //extract this packet's memory address and length in bytes
+        char *mem = mrb->cast<char *>() + offset_bytes;
+        const boost::uint32_t *mem32 = reinterpret_cast<const boost::uint32_t *>(mem);
+        const size_t words32 = (uhd::wtohx(mem32[0]) & 0xffff); //length in words32 (from VRT header)
+        const size_t len = words32*sizeof(boost::uint32_t); //length in bytes
+
+        //check if this receive buffer has been exhausted
+        offset_bytes += len;
+        if (offset_bytes >= mrb->size()) mrb.reset(); //drop caller's ref
+        else if (uhd::wtohx(mem32[words32]) == 0) mrb.reset();
+
+        return make(this, mem, len);
     }
 
 private:
-    const void *get_buff(void) const{return _mem;}
-    size_t get_size(void) const{return _len;}
-
-    bounded_buffer<usb_zero_copy_wrapper_mrb *> &_queue;
-    const void *_mem;
-    size_t _len;
     managed_recv_buffer::sptr _mrb;
+    simple_claimer _claimer;
 };
 
 /***********************************************************************
@@ -63,16 +82,27 @@ private:
 class usb_zero_copy_wrapper_msb : public managed_send_buffer{
 public:
     usb_zero_copy_wrapper_msb(const usb_zero_copy::sptr internal, const size_t fragmentation_size):
-        _internal(internal), _fragmentation_size(fragmentation_size){/*NOP*/}
+        _internal(internal), _fragmentation_size(fragmentation_size)
+    {
+        _ok_to_auto_flush = false;
+        _task = uhd::task::make(boost::bind(&usb_zero_copy_wrapper_msb::auto_flush, this));
+    }
 
-    void commit(size_t len){
-        if (len == 0) return;
+    ~usb_zero_copy_wrapper_msb(void)
+    {
+        //ensure the task has exited before anything auto deconstructs
+        _task.reset();
+    }
+
+    void release(void){
+        boost::mutex::scoped_lock lock(_mutex);
+        _ok_to_auto_flush = true;
 
         //get a reference to the VITA header before incrementing
         const boost::uint32_t vita_header = reinterpret_cast<const boost::uint32_t *>(_mem_buffer_tip)[0];
 
-        _bytes_in_buffer += len;
-        _mem_buffer_tip += len;
+        _bytes_in_buffer += size();
+        _mem_buffer_tip += size();
 
         //extract VITA end of packet flag, we must force flush under eof conditions
         const bool eop = (uhd::wtohx(vita_header) & (0x1 << 24)) != 0;
@@ -80,28 +110,53 @@ public:
         if (eop or full){
             _last_send_buff->commit(_bytes_in_buffer);
             _last_send_buff.reset();
+
+            //notify the auto-flusher to restart its timed_wait
+            lock.unlock(); _cond.notify_one();
         }
     }
 
     UHD_INLINE sptr get_new(const double timeout){
+        boost::mutex::scoped_lock lock(_mutex);
+        _ok_to_auto_flush = false;
+
         if (not _last_send_buff){
             _last_send_buff = _internal->get_send_buff(timeout);
             if (not _last_send_buff) return sptr();
             _mem_buffer_tip = _last_send_buff->cast<char *>();
             _bytes_in_buffer = 0;
         }
-        return make_managed_buffer(this);
+
+        return make(this, _mem_buffer_tip, _fragmentation_size);
     }
 
 private:
-    void *get_buff(void) const{return reinterpret_cast<void *>(_mem_buffer_tip);}
-    size_t get_size(void) const{return _fragmentation_size;}
-
     usb_zero_copy::sptr _internal;
     const size_t _fragmentation_size;
     managed_send_buffer::sptr _last_send_buff;
     size_t _bytes_in_buffer;
     char *_mem_buffer_tip;
+
+    //private variables for auto flusher
+    boost::mutex _mutex;
+    boost::condition_variable _cond;
+    uhd::task::sptr _task;
+    bool _ok_to_auto_flush;
+
+    /*!
+     * The auto flusher ensures that buffers are force committed when
+     * the user has not called get_new() within a certain time window.
+     */
+    void auto_flush(void)
+    {
+        boost::mutex::scoped_lock lock(_mutex);
+        const bool timeout = not _cond.timed_wait(lock, AUTOFLUSH_TIMEOUT);
+        if (timeout and _ok_to_auto_flush and _last_send_buff and _bytes_in_buffer != 0)
+        {
+            _last_send_buff->commit(_bytes_in_buffer);
+            _last_send_buff.reset();
+        }
+    }
 };
 
 /***********************************************************************
@@ -112,44 +167,26 @@ public:
     usb_zero_copy_wrapper(sptr usb_zc, const size_t frame_boundary):
         _internal_zc(usb_zc),
         _frame_boundary(frame_boundary),
-        _available_recv_buffs(this->get_num_recv_frames()),
-        _mrb_pool(this->get_num_recv_frames(), usb_zero_copy_wrapper_mrb(_available_recv_buffs)),
-        _the_only_msb(usb_zero_copy_wrapper_msb(usb_zc, frame_boundary))
+        _next_recv_buff_index(0)
     {
-        BOOST_FOREACH(usb_zero_copy_wrapper_mrb &mrb, _mrb_pool){
-            _available_recv_buffs.push_with_haste(&mrb);
+        for (size_t i = 0; i < this->get_num_recv_frames(); i++){
+            _mrb_pool.push_back(boost::make_shared<usb_zero_copy_wrapper_mrb>());
         }
+        _the_only_msb = boost::make_shared<usb_zero_copy_wrapper_msb>(usb_zc, frame_boundary);
     }
 
     managed_recv_buffer::sptr get_recv_buff(double timeout){
         //attempt to get a managed recv buffer
-        if (not _last_recv_buff.get()){
+        if (not _last_recv_buff){
             _last_recv_buff = _internal_zc->get_recv_buff(timeout);
-            _last_recv_offset = 0;
+            _last_recv_offset = 0; //reset offset into buffer
         }
 
-        //attempt to get a wrapper for a managed recv buffer
-        usb_zero_copy_wrapper_mrb *wmrb = NULL;
-        if (_last_recv_buff.get() and _available_recv_buffs.pop_with_timed_wait(wmrb, timeout)){
-            //extract this packet's memory address and length in bytes
-            const char *mem = _last_recv_buff->cast<const char *>() + _last_recv_offset;
-            const boost::uint32_t *mem32 = reinterpret_cast<const boost::uint32_t *>(mem);
-            const size_t len = (uhd::wtohx(mem32[0]) & 0xffff)*sizeof(boost::uint32_t); //length in bytes (from VRT header)
-
-            managed_recv_buffer::sptr recv_buff; //the buffer to be returned to the user
-            recv_buff = wmrb->get_new(_last_recv_buff, mem, len);
-            _last_recv_offset += len;
-
-            //check if this receive buffer has been exhausted
-            if (_last_recv_offset >= _last_recv_buff->size()) {
-                _last_recv_buff.reset();
-            }
-
-            return recv_buff;
-        }
-
-        //otherwise return a null sptr for failure
-        return managed_recv_buffer::sptr();
+        //get the buffer to be returned to the user
+        if (_next_recv_buff_index == _mrb_pool.size()) _next_recv_buff_index = 0;
+        return _mrb_pool[_next_recv_buff_index]->get_new(
+            _last_recv_buff, _last_recv_offset, timeout, _next_recv_buff_index
+        );
     }
 
     size_t get_num_recv_frames(void) const{
@@ -161,7 +198,7 @@ public:
     }
 
     managed_send_buffer::sptr get_send_buff(double timeout){
-        return _the_only_msb.get_new(timeout);
+        return _the_only_msb->get_new(timeout);
     }
 
     size_t get_num_send_frames(void) const{
@@ -175,16 +212,13 @@ public:
 private:
     sptr _internal_zc;
     size_t _frame_boundary;
-    bounded_buffer<usb_zero_copy_wrapper_mrb *> _available_recv_buffs;
-    std::vector<usb_zero_copy_wrapper_mrb> _mrb_pool;
-    usb_zero_copy_wrapper_msb _the_only_msb;
-
-    //buffer to store partially-received VRT packets in
-    buffer_pool::sptr _fragment_mem;
+    std::vector<boost::shared_ptr<usb_zero_copy_wrapper_mrb> > _mrb_pool;
+    boost::shared_ptr<usb_zero_copy_wrapper_msb> _the_only_msb;
 
     //state for last recv buffer to create multiple managed buffers
     managed_recv_buffer::sptr _last_recv_buff;
     size_t _last_recv_offset;
+    size_t _next_recv_buff_index;
 };
 
 /***********************************************************************
