@@ -126,6 +126,15 @@ public:
 
         _iface->poke32(REG_DSP_TX_INTERP, (hb1 << 9) | (hb0 << 8) | (interp & 0xff));
 
+        if (interp > 1 and hb0 == 0 and hb1 == 0)
+        {
+            UHD_MSG(warning) << boost::format(
+                "The requested interpolation is odd; the user should expect CIC rolloff.\n"
+                "Select an even interpolation to ensure that a halfband filter is enabled.\n"
+                "interpolation = dsp_rate/samp_rate -> %d = (%f MHz)/(%f MHz)\n"
+            ) % interp_rate % (_tick_rate/1e6) % (rate/1e6);
+        }
+
         // Calculate CIC interpolation (i.e., without halfband interpolators)
         // Calculate closest multiplier constant to reverse gain absent scale multipliers
         const double rate_pow = std::pow(double(interp & 0xff), 3);
@@ -136,9 +145,10 @@ public:
     }
 
     void update_scalar(void){
-        const double target_scalar = (1 << 17)*_scaling_adjustment/_dsp_extra_scaling;
+        const double factor = 1.0 + std::max(ceil_log2(_scaling_adjustment), 0.0);
+        const double target_scalar = (1 << 17)*_scaling_adjustment/_dsp_extra_scaling/factor;
         const boost::int32_t actual_scalar = boost::math::iround(target_scalar);
-        _fxpt_scalar_correction = target_scalar/actual_scalar; //should be small
+        _fxpt_scalar_correction = target_scalar/actual_scalar*factor; //should be small
         _iface->poke32(REG_DSP_TX_SCALE_IQ, actual_scalar);
     }
 
