@@ -21,9 +21,6 @@
 #define SSPH_DONT_PAD_TO_ONE
 #include "../../transport/super_send_packet_handler.hpp"
 #include "usrp1_calc_mux.hpp"
-#include "fpga_regs_standard.h"
-#include "fpga_regs_common.h"
-#include "usrp_commands.h"
 #include "usrp1_impl.hpp"
 #include <uhd/utils/msg.hpp>
 #include <uhd/utils/tasks.hpp>
@@ -36,6 +33,26 @@
 #include <boost/format.hpp>
 #include <boost/make_shared.hpp>
 
+#define bmFR_RX_FORMAT_SHIFT_SHIFT 0
+#define bmFR_RX_FORMAT_WIDTH_SHIFT 4
+#define bmFR_TX_FORMAT_16_IQ       0
+#define bmFR_RX_FORMAT_WANT_Q      (0x1  <<  9)
+#define FR_RX_FREQ_0               34
+#define FR_RX_FREQ_1               35
+#define FR_RX_FREQ_2               36
+#define FR_RX_FREQ_3               37
+#define FR_INTERP_RATE             32
+#define FR_DECIM_RATE              33
+#define FR_RX_MUX                  38
+#define FR_TX_MUX                  39
+#define FR_TX_FORMAT               48
+#define FR_RX_FORMAT               49
+#define FR_TX_SAMPLE_RATE_DIV      0
+#define FR_RX_SAMPLE_RATE_DIV      1
+#define GS_TX_UNDERRUN             0
+#define GS_RX_OVERRUN              1
+#define VRQ_GET_STATUS             0x80
+
 using namespace uhd;
 using namespace uhd::usrp;
 using namespace uhd::transport;
@@ -73,8 +90,8 @@ public:
         /* NOP */
     }
 
-    void commit(size_t size){
-        if (size != 0) this->_commit_cb(_curr_buff, _next_buff, size);
+    void release(void){
+        this->_commit_cb(_curr_buff, _next_buff, size());
     }
 
     sptr get_new(
@@ -83,13 +100,13 @@ public:
     ){
         _curr_buff = curr_buff;
         _next_buff = next_buff;
-        return make_managed_buffer(this);
+        return make(this,
+            _curr_buff.buff->cast<char *>() + _curr_buff.offset,
+            _curr_buff.buff->size()         - _curr_buff.offset
+        );
     }
 
 private:
-    void  *get_buff(void) const{return _curr_buff.buff->cast<char *>() + _curr_buff.offset;}
-    size_t get_size(void) const{return _curr_buff.buff->size()         - _curr_buff.offset;}
-
     offset_send_buffer _curr_buff, _next_buff;
     commit_cb_type _commit_cb;
 };
