@@ -1,5 +1,5 @@
 //
-// Copyright 2010-2011 Ettus Research LLC
+// Copyright 2010-2012 Ettus Research LLC
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,22 +26,13 @@
 #include <vector>
 #include <cstdlib> //getenv
 #include <cstdio>  //P_tmpdir
+#ifdef BOOST_MSVC
+#define USE_GET_TEMP_PATH
+#include <windows.h> //GetTempPath
+#endif
 
 namespace fs = boost::filesystem;
 
-/***********************************************************************
- * Determine the paths separator
- **********************************************************************/
-#ifdef UHD_PLATFORM_WIN32
-    static const std::string env_path_sep = ";";
-#else
-    static const std::string env_path_sep = ":";
-#endif /*UHD_PLATFORM_WIN32*/
-
-#define path_tokenizer(inp) \
-    boost::tokenizer<boost::char_separator<char> > \
-    (inp, boost::char_separator<char>(env_path_sep.c_str()))
-
 /***********************************************************************
  * Get a list of paths for an environment variable
  **********************************************************************/
@@ -52,6 +43,20 @@ static std::string get_env_var(const std::string &var_name, const std::string &d
 
 static std::vector<fs::path> get_env_paths(const std::string &var_name){
 
+/***********************************************************************
+ * Determine the paths separator
+ **********************************************************************/
+
+#ifdef UHD_PLATFORM_WIN32
+    static const std::string env_path_sep = ";";
+#else
+    static const std::string env_path_sep = ":";
+#endif /*UHD_PLATFORM_WIN32*/
+
+#define path_tokenizer(inp) \
+    boost::tokenizer<boost::char_separator<char> > \
+    (inp, boost::char_separator<char>(env_path_sep.c_str()))
+
     std::string var_value = get_env_var(var_name);
 
     //convert to filesystem path, filter blank paths
@@ -67,19 +72,20 @@ static std::vector<fs::path> get_env_paths(const std::string &var_name){
 /***********************************************************************
  * Get a list of special purpose paths
  **********************************************************************/
-static fs::path get_uhd_pkg_data_path(void){
-    return fs::path(get_env_var("UHD_PKG_DATA_PATH", UHD_PKG_DATA_PATH));
+std::string uhd::get_pkg_data_path(void)
+{
+    return get_env_var("UHD_PKG_DATA_PATH", UHD_PKG_DATA_PATH);
 }
 
 std::vector<fs::path> get_image_paths(void){
     std::vector<fs::path> paths = get_env_paths("UHD_IMAGE_PATH");
-    paths.push_back(get_uhd_pkg_data_path() / "images");
+    paths.push_back(fs::path(uhd::get_pkg_data_path()) / "images");
     return paths;
 }
 
 std::vector<fs::path> get_module_paths(void){
     std::vector<fs::path> paths = get_env_paths("UHD_MODULE_PATH");
-    paths.push_back(get_uhd_pkg_data_path() / "modules");
+    paths.push_back(fs::path(uhd::get_pkg_data_path()) / "modules");
     return paths;
 }
 
@@ -87,13 +93,35 @@ std::vector<fs::path> get_module_paths(void){
  * Implement the functions in paths.hpp
  **********************************************************************/
 std::string uhd::get_tmp_path(void){
-    const char *tmp_path = std::getenv("TMP");
+    const char *tmp_path = NULL;
+
+    //try the official uhd temp path environment variable
+    tmp_path = std::getenv("UHD_TEMP_PATH");
     if (tmp_path != NULL) return tmp_path;
 
+    //try the windows function if available
+    #ifdef USE_GET_TEMP_PATH
+    char lpBuffer[2048];
+    if (GetTempPath(sizeof(lpBuffer), lpBuffer)) return lpBuffer;
+    #endif
+
+    //try windows environment variables
+    tmp_path = std::getenv("TMP");
+    if (tmp_path != NULL) return tmp_path;
+
+    tmp_path = std::getenv("TEMP");
+    if (tmp_path != NULL) return tmp_path;
+
+    //try the stdio define if available
     #ifdef P_tmpdir
     if (P_tmpdir != NULL) return P_tmpdir;
     #endif
 
+    //try unix environment variables
+    tmp_path = std::getenv("TMPDIR");
+    if (tmp_path != NULL) return tmp_path;
+
+    //give up and use the unix default
     return "/tmp";
 }
 
