@@ -1,5 +1,5 @@
 //
-// Copyright 2010-2011 Ettus Research LLC
+// Copyright 2010-2012 Ettus Research LLC
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -33,5 +33,9 @@ UHD_STATIC_BLOCK(print_system_info){
 #endif
 
 std::string uhd::get_version_string(void){
-    return "@UHD_VERSION@-@UHD_BUILD_INFO@";
+    return "@UHD_VERSION@";
+}
+
+std::string uhd::get_abi_string(void){
+    return UHD_VERSION_ABI_STRING;
 }
